/**
 * editor_plugin_src.js
 *
 * Copyright 2009, Moxiecode Systems AB
 * Released under LGPL License.
 *
 * License: http://tinymce.moxiecode.com/license
 * Contributing: http://tinymce.moxiecode.com/contributing


 * completly changed and extended for tinymce 6 mirko boeer 2023

 */



if(typeof GetEMailCleanedHTML !== "function"){
    var script = document.createElement("script");
    var uri = tinymce.util.URI.getDocumentBaseUrl(document.location) + 'tinymce/';
    script.src = uri + "htmlcleaner.php?nocache=" + (new Date().getTime() / 1000);
    document.head.appendChild(script);  // add it to the end of the head section of the page (could change 'head' to 'body' to add it to the end of the body section instead)
}

tinymce.PluginManager.add('fullpagemb', (editor, url) => {

  const FPclass = {
    fp_beforehead: "",
    fp_afterhead: "",
    fp_head: "",
    fp_foot: "",

    _beforesetContent: function (contents){
      var startPos, endPos, headerFragment, styles = '', dom = editor.dom, elm;

      if(contents.target && contents.target.plugins && contents.target.plugins.initialization){
         // add attributs for target groups,
         // add IPE Tags and attributs
         // => tinymce removes unknown attributs
         AddOwnHTMLTagsAndAttribs(contents.target);
      }

      if( contents.format == "html" && (contents.paste && contents.paste == true) ||
        (contents.merge && contents.merge == true) ){
        contents.content = CheckSecurityOfPastedHTMLCode(contents.content, editor);
      }

      if(contents.format == "text" ||
        (contents.selection && contents.selection == true) ||
        (contents.no_selection && contents.no_selection == true) ||
        (contents.paste && contents.paste == true) ||
        (contents.merge && contents.merge == true)
        )
          return; //link dialog URL, insert elements, undo/redo, paste images

      if(!contents.initial && !contents.selection && contents.set && contents.content == ""){ // file - new

        this.fp_beforehead = '<!DOCTYPE html>\n<html>';
        this.fp_afterhead = "<body>";
        this.fp_head = '<meta name="viewport" content="width=device-width, initial-scale=1"><meta name="format-detection" content="telephone=no"><title></title><meta http-equiv="X-UA-Compatible" content="IE=edge">';
        this.fp_foot = '\n</body>\n</html>';

        dom.remove('fullpage_styles');

        let b = editor.getBody();
        dom.setAttribs(b, {
         bgColor : '',
         bgimage : ''
        });

        // call templates?

        return;
      }

      //if (contents.format == 'raw' && this.fp_head)
      //  return;

      function low(s) {
       return s.replace(/<\/?[A-Z]+/g, function(a) {
        return a.toLowerCase();
       })
      };

      var content = "";
      let bodyIncluded = false;
      if(contents.initial && contents.initial == true && contents.content.search(/<body/i) < 0){  //FF
        content = contents.target && contents.target.targetElm && contents.target.targetElm.innerHTML ? contents.target.targetElm.innerHTML : "";
        if(content == "")
           content = document.getElementById( contents.target.id ).innerHTML;
        content = content.replace(/&lt;/g, '<').replace(/&gt;/g, '>'); // replace < > from entities;

      }else{
        content = contents.content;
        bodyIncluded = true;
      }

      content = content.replace(/<(\/?)BODY/gi, '<$1body');

      startPos = content.lastIndexOf('<body');

      if (startPos != -1) {
       startPos = content.indexOf('>', startPos);
       this.fp_head = low(content.substring(0, startPos + 1));

       endPos = content.lastIndexOf('</body');
       if (endPos == -1)
         endPos = content.length;

       if(bodyIncluded){
         contents.content = content.substring(startPos + 1, endPos);
       }
       this.fp_foot = low(content.substring(endPos));

       if(contents.content.indexOf('<body') > -1)
          tinymce.activeEditor.notificationManager.open({
            text: 'Mehrere body-Tags werden nicht unterst&uuml;tzt, verwenden Sie den reinen Quelltext-Modus f&uuml;r solche speziellen Inhalte.',
            type: 'alert'
          });

      } else {
       this.fp_head = '<!DOCTYPE html>\n<html><head><meta name="viewport" content="width=device-width, initial-scale=1"><meta name="format-detection" content="telephone=no"><title></title><meta http-equiv="X-UA-Compatible" content="IE=edge"></head>\n<body>';
       this.fp_foot = '\n</body>\n</html>';
      }

      // Parse header and update iframe
      headerFragment =  tinymce.html.DomParser({
                           validate: false,
                           root_name: '#document'
                          }).parse(this.fp_head);

      tinymce.util.Tools.each(headerFragment.getAll('style'), function(node) {
       if (node.firstChild)
        styles += node.firstChild.value;
      });

      // split it in real header, before header and after header parts
      var x = this.fp_head.split(/<head.*?>/gi, 2);
      if(x.length > 1){
        this.fp_beforehead = x[0];
        var y = x[1].split(/<\/head.*?>/gi, 2);
        if(y.length > 1){
          this.fp_afterhead = y[1];
          this.fp_head = x[1].split(/<\/head.*?>/gi, 2)[0].trim();
        }else{
          this.fp_beforehead = "";
        }
      }

      if(!this.fp_beforehead)
        this.fp_beforehead = '<!DOCTYPE html>\n<html>';
      if(!this.fp_afterhead)
        this.fp_afterhead = "<body>";
      if(!this.fp_foot)
        this.fp_foot = '\n</body>\n</html>';


      var t = this.fp_afterhead.trim();
      t = t.replace(/<body /ig, "<table ").replace(/<body/ig, "<table");
      var myBody = document.createElement('comment');
      myBody.innerHTML = t;
      elm = myBody && myBody.firstChild;

      if (elm && typeof elm.getAttribute === 'function') {
       let b = editor.getBody();
       dom.setAttribs(b, {
        style : elm.getAttribute('style') || '',
        dir : elm.getAttribute('dir') || '',
        vLink : elm.getAttribute('vlink') || '',
        link : elm.getAttribute('link') || '',
        aLink : elm.getAttribute('alink') || '',
        bgColor : elm.getAttribute('bgColor') || '',
        bgimage : elm.getAttribute('bgimage') || '',
        leftMargin : elm.getAttribute('leftMargin') || '',
        topMargin : elm.getAttribute('topMargin') || ''
       });

       b.style.backgroundColor = elm.style.backgroundColor;
       b.style.backgroundImage = elm.style.backgroundImage;
       b.style.color = elm.style.color;

      }
      myBody.remove();

      dom.remove('fullpage_styles');

      var firstHeadInEditor = editor.getDoc().getElementsByTagName("head")[0];

      tinymce.util.Tools.each(firstHeadInEditor.getElementsByTagName("link"), function(node) {
          if( node.rel && node.rel === "stylesheet" && node.getAttribute("data-mce-fullpage") )
             node.parentNode.removeChild(node);
      });

      tinymce.util.Tools.each(headerFragment.getAll('link'), function(node) {
        if(!node.attr("href") || !node.attr("rel") || node.attr("rel") !== "stylesheet") return 1;

        dom.add(firstHeadInEditor, "link", {
                rel: "stylesheet",
                text: "text/css",
                href: node.attr("href"),
                "data-mce-fullpage": "1"
            });
      });

      //bgsound??

      if (styles) {
       dom.add(firstHeadInEditor, 'style', {id : 'fullpage_styles'}, styles);

       // Needed for IE 6/7
       elm = dom.get('fullpage_styles');
       if (elm.styleSheet)
        elm.styleSheet.cssText = styles;
      }

    },

    _getContent: function (contents){

      if(contents.format == "text") return;
      //contents.source_view true on sourceview
      //contents.selection true on selection
      //contents.save on save

      // insert content
      if(typeof contents.selection !== "undefined" || typeof contents.no_selection !== "undefined") return;

      if(editor.getParam("fullpage", false) === true){
        var head = tinymce.trim(this.fp_head);
        if(head.indexOf('<head') < 0)
          head = "<head>" + head + "</head>";
        contents.content = tinymce.trim(this.fp_beforehead) + head + this.fp_afterhead + '\n' + tinymce.trim(contents.content) + '\n' + tinymce.trim(this.fp_foot);
        if(contents.format !== "raw" && editor.getParam("fullpage_EMailCleaner", false) == true)
           contents.content = GetEMailCleanedHTML( contents.content, null, editor.config.basePath, editor.config.baseUserFilesPath );
      }else{
           contents.content = GetCleanedHTML( contents.content, null );
      }
    }

  }

  if(editor.getParam("fullpage", false) === true){
    editor.on('BeforeSetContent', FPclass._beforesetContent);
  }
  editor.on('GetContent', FPclass._getContent);

  function mceFullPagePropertiesmb() {


           function removeUrl(src){
             if(src.indexOf("url(&quot;") != -1) {
                src = src.replace("url(&quot;", "url('");
                src = src.replace("&quot;)", "')");
             }
             src = src.trim();
             if(src.indexOf("url(") != -1) {
                src = src.replace("url(", "");
                src = src.replace("')", "");
                src = src.replace("\")", "");
                src = src.trim();
               if(src[0] == "'" || src[0] == '"')
                 src = src.substr(1);
               if(src[src.length - 1] == "'" || src[src.length - 1] == '"')
                 src = src.substr(0, src.length - 1);
             }
             return src;
           }

         // my full page body, insert it as table to modify properties
         var t = editor.fp_afterhead.trim();
         t = t.replace(/<body/ig, "<table");
         var myBody = document.createElement('comment');
         myBody.innerHTML = t;


         var comment = document.createElement('comment');
         comment.innerHTML = editor.fp_head;

         var meta = {};
         var metas = comment.getElementsByTagName('meta');
         for(var j=0; j<metas.length; j++) {
           if(metas[j].name)
             meta[metas[j].name] = metas[j];
         }

         const body = editor.getBody();
         const cStyle = getComputedStyle(body);

         var dialogTitle = editor.options.get("language") == "de" ? "Seiten-/Body-Eigenschaften" : "Page-/ Body properties";

         // https://www.tiny.cloud/docs/tinymce/6/dialog/
         editor.windowManager.open({
                title: dialogTitle,
                size: "normal",
                body: {
                    type: "panel",
                    items: [
                      {
                        type: 'colorinput',
                        name: 'textcolor',
                        label: "Standard Textfarbe"
                      },
                      {
                        type: 'colorinput',
                        name: 'bgcolor',
                        label: "Hintergrundfarbe"
                      },
                      {
                          name: "bgimage",
                          type: "urlinput",
                          label: "Hintergrundbild",
                          filetype: 'image'
                      },

                      {
                         name: "description",
                         type: "input",
                         label: "Beschreibung"
                      },
                      {
                         name: "copyright",
                         type: "input",
                         label: "Copyright"
                      },
                      {
                         name: "author",
                         type: "input",
                         label: "Autor"
                      }

                    ]
                },
                buttons: [{
                    type: "cancel",
                    name: "cancel",
                    text: "Cancel"
                }, {
                    type: "submit",
                    name: "save",
                    text: "Save",
                    primary: true
                }],
                 initialData: {
                     textcolor: cStyle.color ? _convertRGBToHex(cStyle.color) : '#000000',
                     bgcolor: body.bgColor ? _convertRGBToHex(body.bgColor) : body.style.backgroundColor ? _convertRGBToHex(body.style.backgroundColor) : '#ffffff',
                     bgimage: { value: ( body.background ? body.background : (body.style.backgroundImage ? removeUrl(body.style.backgroundImage) : "") ) },
                     description: meta["description"] && meta["description"].content ? meta["description"].content : "",
                     copyright: meta["copyright"] && meta["copyright"].content ? meta["copyright"].content : "",
                     author: meta["author"] && meta["author"].content ? meta["author"].content : ""
                  },
                  onCancel: function(api){
                    comment = null;
                    myBody = null;
                  },
                  onSubmit: function(api) {
                    const data = api.getData();

                    body.bgColor = _convertRGBToHex(data.bgcolor);
                    myBody.firstChild.bgColor = _convertRGBToHex(data.bgcolor);
                    body.style.backgroundColor = _convertRGBToHex(data.bgcolor);
                    myBody.firstChild.style.backgroundColor = _convertRGBToHex(data.bgcolor);
                    body.style.color = _convertRGBToHex(data.textcolor);
                    myBody.firstChild.style.color = _convertRGBToHex(data.textcolor);
                    body.background = data.bgimage.value;
                    myBody.firstChild.background = data.bgimage.value;
                    if(data.bgimage.value === ""){
                       body.style.backgroundImage = "";
                       myBody.firstChild.style.backgroundImage = "";
                    }
                    else{
                       body.style.backgroundImage = "url('" +data.bgimage.value + "')";
                       myBody.firstChild.style.backgroundImage = "url('" +data.bgimage.value + "')";
                    }

                    if(data.description){
                      if( meta["description"] )
                         meta["description"].content = data.description;
                         else{
                           var e = document.createElement('meta');
                           e.setAttribute('content', data.description);
                           e.setAttribute('name', "description");
                           comment.append(e);
                         }
                    }else
                      if( meta["description"] )
                        meta["description"].remove();

                    if(data.copyright){
                      if( meta["copyright"] )
                         meta["copyright"].content = data.copyright;
                         else{
                           var e = document.createElement('meta');
                           e.setAttribute('content', data.copyright);
                           e.setAttribute('name', "copyright");
                           comment.append(e);
                         }
                    }else
                      if( meta["copyright"] )
                        meta["copyright"].remove();

                    if(data.author){
                      if( meta["author"] )
                         meta["author"].content = data.author;
                         else{
                           var e = document.createElement('meta');
                           e.setAttribute('content', data.author);
                           e.setAttribute('name', "author");
                           comment.append(e);
                         }
                    }else
                      if( meta["author"] )
                        meta["author"].remove();

                    editor.fp_head = comment.innerHTML;

                    comment = null;

                    var t = myBody.innerHTML;
                    t = t.replace(/<\/table>/ig, "");
                    t = t.replace(/<table /ig, "<body ");
                    editor.fp_afterhead = t.trim();
                    myBody = null;

                    editor.setDirty(true);

                    api.close();
                }
            })

  };


  editor.ui.registry.addIcon('pageproperties', DownloadFile(tinymce.defaulteditorConfig.baseMyPluginsUrl + 'fullpagemb/pageproperties.svg'));

  if(editor.getParam("fullpage", false) === true){
    editor.ui.registry.addButton('btnPageBodyProperties', {
      icon: "pageproperties",
      //text: 'Document Properties',
      tooltip: "Seiten-/Body Eigenschaften",
      onAction: mceFullPagePropertiesmb
    });

    editor.ui.registry.addMenuItem('mnuPageBodyProperties', {
      text: 'Seiten-/Body Eigenschaften...',
      icon: "pageproperties",
      onAction: () => {
        mceFullPagePropertiesmb();
      }
    });

    editor.ui.registry.addContextMenu('mnuPageBodyProperties', {
       update: (element) => ['BODY', 'HTML'].contains(element.tagName) || element.parentElement && element.parentElement.tagName == "BODY" ? "mnuPageBodyProperties" : ""
     });

  }

  return {
    name: 'FullPageEditPlugin',
    url: 'https://www.superwebmailer.de/'
  };

});


if(typeof _convertRGBToHex === 'undefined')
    function _convertRGBToHex( cssStyle )
    {
     if(cssStyle == null)
       return "";
      if(cssStyle.indexOf("rgb") < 0)
        return cssStyle;

     if(cssStyle.indexOf("rgba") < 0) // chrome rgba
       return cssStyle.replace( /(?:rgb\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\))/gi, function( match, red, green, blue )
        {
         red = parseInt( red, 10 ).toString( 16 );
         green = parseInt( green, 10 ).toString( 16 );
         blue = parseInt( blue, 10 ).toString( 16 );
         var color = [red, green, blue] ;

         // Add padding zeros if the hex value is less than 0x10.
         for ( var i = 0 ; i < color.length ; i++ )
          color[i] = String( '0' + color[i] ).slice( -2 ) ;

         return '#' + color.join( '' ) ;
         });


       return cssStyle.replace( /(?:rgba\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\))/gi, function( match, red, green, blue, alpha )
        {
         red = parseInt( red, 10 ).toString( 16 );
         green = parseInt( green, 10 ).toString( 16 );
         blue = parseInt( blue, 10 ).toString( 16 );
         var color = [red, green, blue] ;

         // Add padding zeros if the hex value is less than 0x10.
         for ( var i = 0 ; i < color.length ; i++ )
          color[i] = String( '0' + color[i] ).slice( -2 ) ;

         if(alpha == 0 && red == 0 && green == 0 && blue == 0)
           return 'transparent';
         else
           return '#' + color.join( '' ) ;
         });

    }
